part of tecfy_basic_package;

abstract class BaseController {
  /// bust feild
  bool busy = false;

  /// on dispose
  void dispose() {}

  /// the state
  State? state;

  late BuildContext _context;

  /// the local function for setState
  void Function(VoidCallback)? setStateFunction;

  /// buld context for the controller
  BuildContext get context {
    return state?.context ?? _context;
  }

  /// base controller
  BaseController(this.state, {buildContext})
      : assert(state != null || buildContext != null) {
    _context = buildContext;
  }

  /// navigate to other page
  navigateTo(creator,
      [bool replacment = false,
      AnimationTypes animationType = AnimationTypes.slideLeft]) {
    return AppNavigator.navigateTo(context, creator,
        replacment: replacment, animationType: animationType);
  }

  /// setState
  setState() {
    if (setStateFunction != null)
      setStateFunction?.call(() {});
    else if (state != null && (state?.mounted ?? false)) state?.setState(() {});
  }

  /// show message
  Future<dynamic> showMessage(String title,
      {String heroTag = 'submit_button',
      MessageType type = MessageType.none,
      String? message,
      String? imageAssert}) async {
    if (busy) return;
    busy = true;
    await AppNavigator.showMessage(context, title,
        heroTag: heroTag,
        type: type,
        message: message,
        imageAssert: imageAssert);
    busy = false;
  }

  /// confirm message with Yes and No buttons
  Future<bool> confirmMessage(String title,
      {String? message, String? imageAssert}) async {
    if (busy) return false;
    busy = true;
    var result = await AppNavigator.confirmMessage(context, title,
        message: message, imageAssert: imageAssert);
    busy = false;
    return result;
  }

  /// show loading popup
  setLoading() async {
    if (busy) return;
    busy = true;
    await AppNavigator.showLoading(context);
    busy = false;
  }

  /// dismiss the loading popup
  setResume() {
    if (!busy) return;
    busy = false;
    Navigator.of(context, rootNavigator: true).pop();
  }

  /// exec haptic feedback
  Future<void> hapticFeedback({bool heavy = true}) {
    return AppNavigator.hapticFeedback(heavy: heavy);
  }
}
